import { CronJob } from "cron";
import { v4 } from "uuid";

interface Timer {
    id: string;
    job: CronJob;
    context: Map<string, any>;
}

export default class TimerHelper {
    private _timers: Timer[];

    constructor() {
        this._timers = [];
    }

    public AddTimer(
        cronTime: string,
        timeZone: string,
        onTick: ((context: Map<string, any>) => void) | ((context: Map<string, any>) => Promise<void>)): string {
        const context = new Map<string, any>();

        const job = new CronJob(
            cronTime,
            () => {
                onTick(context);
            },
            null,
            false,
            timeZone,
        );

        const id = v4();

        this._timers.push({
            id,
            job,
            context,
        });

        return id;
    }

    public StartAllTimers() {
        this._timers.forEach(timer => timer.job.start());
    }

    public StopAllTimers() {
        this._timers.forEach(timer => timer.job.stop());
    }

    public StartTimer(id: string) {
        const timer = this._timers.find(x => x.id == id);

        if (!timer) return;

        timer.job.start();
    }

    public StopTimer(id: string) {
        const timer = this._timers.find(x => x.id == id);

        if (!timer) return;

        timer.job.stop();
    }
}